
/*
   libmikmod for xmms

   Modified mikmod lib 2/1/99 for xmms by J. Nick Koston (BlueDraco)

   Current Version 0.9.9

   Changes: (1) 2/1/99 Inital Build / Release
   (2) 2/2/99 Psychotron fixed a small problem w/interfacing
   (3) 2/2/99 Shows Channels in spectrum analyzer
   (4) 2/2/99 Fixed the infinate loop problem, and the absured channel
   value problem, removed the mikmod_going stuff
   (5) 2/2/99 Added back in mikmod_going (it is needed)
   (6) 2/2/99 Repaired SEGV on playlist goes to next song, bad
   printf when the mod cannot be loaded and xmms is in repeat mode.
   Also made the absured channel display code work 
   (7) 2/5/99 Removed some old useless stuff
   (8) 2/5/99 Added about -- (doesn't work .. not in xmms yet)
   (9) 2/8/99 Added configure -- (about and configure work now w/a2)

 */

#include "xmms/i18n.h"
#include "mikmod-plugin.h"
#include "libxmms/configfile.h"
#include "libxmms/util.h"
#include "libxmms/titlestring.h"
#include <gtk/gtk.h>

#include "mikmod.xpm"

static void init(void);
static int is_our_file(char *filename);
extern InputPlugin mikmod_ip;
static void play_file(char *filename);
static int get_time(void);
static void stop(void);
static void *play_loop(void *arg);
static void mod_pause(short p);
static void seek(int time);
static void aboutbox(void);
static void get_song_info(char *filename, char **title, int *length);
static void configure();
static void config_ok(GtkWidget * widget, gpointer data);

static pthread_t decode_thread;

MIKMODConfig mikmod_cfg;
extern gboolean mikmod_xmms_audio_error;

/* current module */
static MODULE *mf = NULL;

/* module parameters */
static gboolean cfg_extspd = 1;	/* Extended Speed enable */
static gboolean cfg_panning = 1;	/* DMP panning enable (8xx effects) */
static gboolean cfg_wrap = 0;	/* auto song-wrapping disable */
static gboolean cfg_loop = 0;	/* allow module to loop */
gboolean mikmod_going = 1;









static void *play_loop(void *arg);
static void config_ok(GtkWidget * widget, gpointer data);


static void aboutbox()
{
}

static void stop(void)
{
	if (mikmod_going)
	{
		mikmod_going = 0;
		pthread_join(decode_thread, NULL);
	}
}

static void seek(int time)
{
	/*
	   We need to seek in pattrens somehow
	   can't seek by time only by X pattrens on way or the other

	   Player_NextPosition ();
	   Player_PrevPosition ();

	 */
	return;
}

static void mod_pause(short p)
{
	mikmod_ip.output->pause(p);
}

static int get_time(void)
{
	if (mikmod_xmms_audio_error)
		return -2;
	if (!mikmod_going)
		return -1;
	if(!Player_Active() && !mikmod_ip.output->buffer_playing())
		return -1;
	return mikmod_ip.output->output_time();
}

InputPlugin *get_iplugin_info(void)
{
	mikmod_ip.description = g_strdup_printf(_("MikMod Player %s"), VERSION);
	return &mikmod_ip;
}

static void init(void)
{

	ConfigFile *cfg;

	md_device = 0;		/* standard device: autodetect */
	md_reverb = 0;		/* Reverb (max 15) */

	mikmod_cfg.mixing_freq = SAMPLE_FREQ_44;
	mikmod_cfg.volumefadeout = 0;
	mikmod_cfg.surround = 0;
	mikmod_cfg.force8bit = 0;
	mikmod_cfg.hidden_patterns = 0;
	mikmod_cfg.force_mono = 0;
	mikmod_cfg.interpolation = TRUE;
	mikmod_cfg.def_pansep = 64;

	if ((cfg = xmms_cfg_open_default_file()) != NULL)
	{
		xmms_cfg_read_int(cfg, "MIKMOD", "mixing_freq", &mikmod_cfg.mixing_freq);
		xmms_cfg_read_int(cfg, "MIKMOD", "volumefadeout", &mikmod_cfg.volumefadeout);
		xmms_cfg_read_int(cfg, "MIKMOD", "surround", &mikmod_cfg.surround);
		xmms_cfg_read_int(cfg, "MIKMOD", "force8bit", &mikmod_cfg.force8bit);
		xmms_cfg_read_int(cfg, "MIKMOD", "hidden_patterns", &mikmod_cfg.hidden_patterns);
		xmms_cfg_read_int(cfg, "MIKMOD", "force_mono", &mikmod_cfg.force_mono);
		xmms_cfg_read_int(cfg, "MIKMOD", "interpolation", &mikmod_cfg.interpolation);

		xmms_cfg_read_int(cfg, "MIKMOD", "default_panning", &mikmod_cfg.def_pansep);
		xmms_cfg_free(cfg);
	}
	MikMod_RegisterAllLoaders();
	MikMod_RegisterDriver(&drv_xmms);
}

static int is_our_file(char *filename)
{
	char *ext, *tmp;

	tmp = strrchr(filename, '/');
	ext = strrchr(filename, '.');
	
	if (ext)
	{
		if (!strcasecmp(ext, ".669"))
			return 1;
		if (!strcasecmp(ext, ".amf"))
			return 1;
		if (!strcasecmp(ext, ".dsm"))
			return 1;
		if (!strcasecmp(ext, ".far"))
			return 1;
		if (!strcasecmp(ext, ".it"))
			return 1;
		if (!strcasecmp(ext, ".m15"))
			return 1;
		if (!strcasecmp(ext, ".med"))
			return 1;
		if (!strcasecmp(ext, ".mod"))
			return 1;
		if (!strcasecmp(ext, ".mtm"))
			return 1;
		if (!strcasecmp(ext, ".s3m"))
			return 1;
		if (!strcasecmp(ext, ".stm"))
			return 1;
		if (!strcasecmp(ext, ".ult"))
			return 1;
		if (!strcasecmp(ext, ".xm"))
			return 1;
		if (!strcasecmp(ext, ".imf"))
			return 1;
		if (!strcasecmp(ext, ".gdm"))
			return 1;
		if (!strcasecmp(ext, ".stx"))
			return 1;
	}

	if (tmp)
	{
		if(!strncasecmp("/mod.", tmp, 5))
			return 1;
	}

	return 0;
}

static gchar *get_title(gchar *filename)
{
	TitleInput *input;
	gchar *temp, *ext, *title;

	if((temp = Player_LoadTitle(filename)) != NULL)
		title = g_strdup(temp);
	else
	{
		XMMS_NEW_TITLEINPUT(input);

		temp = g_strdup(filename);
		ext = strrchr(temp, '.');
		if (ext)
			*ext = '\0';
		input->file_name = g_basename(temp);
		input->file_ext = ext ? ext+1 : NULL;
		input->file_path = temp;

		title = xmms_get_titlestring(xmms_get_gentitle_format(),
					     input);
		if ( title == NULL )
			title = g_strdup(input->file_name);

		g_free(temp);
		g_free(input);
	}

	return title;
}

static void get_song_info(char *filename, char **title, int *length)
{
	(*title) = get_title(filename);
	(*length) = -1;
}

static void play_file(char *filename)
{
	int cfg_maxchn = 32;
	int uservolume = 128;
	int channelcnt = 1;
	int format = FMT_U8;
	FILE *f;
	
	if(!(f = fopen(filename,"rb")))
	{
		mikmod_going = 0;
		return;
	}
	fclose(f);

	mikmod_xmms_audio_error = FALSE;
	mikmod_going = 1;

	switch (mikmod_cfg.mixing_freq)
	{
		case SAMPLE_FREQ_22:
			md_mixfreq = 22050;	/* 1:2 mixing freq */
			break;
		case SAMPLE_FREQ_11:
			md_mixfreq = 11025;	/* 1:4 mixing freq */
			break;
		default:
			md_mixfreq = 44100;	/* standard mixing freq */
			break;
	}

	md_mode = DMODE_SOFT_MUSIC | DMODE_HQMIXER;
	if ((mikmod_cfg.surround == 1))
	{
		md_mode |= DMODE_SURROUND;
	}
	if ((mikmod_cfg.force8bit == 0))
	{
		format = FMT_S16_NE;
		md_mode |= DMODE_16BITS;
	}
	if ((mikmod_cfg.force_mono == 0))
	{
		channelcnt = DMODE_STEREO;
		md_mode |= DMODE_STEREO;
	}
	if ((mikmod_cfg.interpolation == 1))
	{
		md_mode |= DMODE_INTERP;
	}
	md_pansep = mikmod_cfg.def_pansep;
	
#if (LIBMIKMOD_VERSION > 0x30106)
	MikMod_Init("");
#else
	MikMod_Init();
#endif
	if (!(mf = Player_Load(filename, cfg_maxchn, mikmod_cfg.hidden_patterns)))
	{
		mikmod_ip.set_info_text(_("Couldn't load mod"));
		mikmod_going = 0;
		return;
	}
	mf->extspd = cfg_extspd;
	mf->panflag = cfg_panning;
	mf->wrap = cfg_wrap;
	mf->loop = cfg_loop;
	mf->fadeout = mikmod_cfg.volumefadeout;

	Player_Start(mf);
	
	if (mf->volume > uservolume)
		Player_SetVolume(uservolume);

/* mods are in pattrens .. you need to be able to seek
   back the forth from pattrens */

	mikmod_ip.set_info(mf->songname, -1, ((mf->bpm * 1000)), md_mixfreq, channelcnt);
	pthread_create(&decode_thread, NULL, play_loop, NULL);
	return;

}

static void *play_loop(void *arg)
{
	while (mikmod_going)
	{
		if (Player_Active())
			drv_xmms.Update();
		else
		{
			xmms_usleep(10000);
		}

	}
	Player_Stop();		/* stop playing */
	Player_Free(mf);	/* and free the module */

	mikmod_going = 0;
	MikMod_Exit();
	pthread_exit(NULL);
}

static void
     configure()
{
}

static void config_ok(GtkWidget * widget, gpointer data)
{

}

InputPlugin mikmod_ip =
{
	NULL,
	NULL,
	NULL, /* Description */
	init,
	aboutbox,
	configure,
	is_our_file,
	NULL,
	play_file,
	stop,
	mod_pause,
	seek,
	NULL,
	get_time,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	get_song_info,
	NULL,			/* file_info_box */
	NULL
};
