/*  XMMS - Cross-platform multimedia player
 *  Copyright (C) 1998-2000  Peter Alm, Mikael Alm, Olle Hallnas, Thomas Nilsson and 4Front Technologies
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
#define RBSIZE 1024
#include <sys/types.h>
#include <sys/wait.h>
#include <unistd.h>
#include <fcntl.h>
#include <signal.h>
#include <assert.h>
#include "wav.h"
#include "libxmms/util.h"
#include "libxmms/titlestring.h"
#include "libxmms/configfile.h"
#include "xmms/i18n.h"
#include "xmms/plugin.h"
#include "xmms/eoss.h"
#include <pthread.h>
#include <string.h>

InputPlugin mpl_ip =
{
	NULL,
	NULL,
	NULL, /* Description */
	mplayer_init,
	NULL,
	NULL,
	mis_our_file,
	NULL,
	mplay_file,
	mstop,
	mwav_pause,
	mseek,
	NULL,
	mget_time,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	mget_song_info,
	NULL,			/* file_info_box */
	NULL
};

extern OutputPlugin* get_current_output_plugin(void);


static gboolean fs=FALSE;
static char *nowplaying=NULL;
static int spos;
static char statusline[RBSIZE],linebuffer[RBSIZE];
static char *readpos,*writepos;
static pthread_t decode_thread;
static gboolean audio_error = FALSE;
static gboolean playing=FALSE;
static int seeking=0;
static 	char *argval[26];
static	int child;
static int filedesin[2]={-1,-1};
static int filedesout[2]={-1,-1};
static gboolean lengthvalid=FALSE;
static int control,feedback;
static char reply[8192];
static long int atime,vtime;

InputPlugin *get_iplugin_info(void)
{

	mpl_ip.description = g_strdup_printf(_("MPlayer plugin%s"), VERSION);
	return &mpl_ip;
}

static void mplayer_init(void)
{
    ConfigFile *cfg;
    
    readpos=writepos=statusline;
    
    if ((cfg = xmms_cfg_open_default_file()) != NULL)
    {
	xmms_cfg_read_boolean(cfg, "MPLAYER", "fullscreendefault", &fs);    
    }

}

/* needed for is_our_file() */
static int mis_our_file(char *filename)
{
	gchar *ext;
	ext = strrchr(filename, '.');
	if (ext)
	{
		if (!strcasecmp(ext, ".avi"))
			return TRUE;	
		if (!strcasecmp(ext, ".mpg"))
			return TRUE;
		if (!strcasecmp(ext, ".mpeg"))
			return TRUE;
		if (!strcasecmp(ext, ".vob"))
			return TRUE;
		if (!strcasecmp(ext, ".asf"))
			return TRUE;
		if (!strcasecmp(ext, ".wmv"))
			return TRUE;
//		if (!strcasecmp(ext, ".wma"))
//			return TRUE;
		if (!strcasecmp(ext, ".mov"))
			return TRUE;
		if (!strcasecmp(ext, ".ogm"))
			return TRUE;
		if (!strcasecmp(ext, ".asx"))
			return TRUE;
		if (!strcasecmp(ext, ".rm"))
			return TRUE;
			
	}
	return FALSE;
}

static gchar *mget_title(gchar *filename)
{
	TitleInput *input;
	gchar *temp, *ext, *title;

	XMMS_NEW_TITLEINPUT(input);

	temp = g_strdup(filename);
	ext = strrchr(temp, '.');
	if (ext)
		*ext = '\0';
	input->file_name = g_basename(temp);
	input->file_ext = ext ? ext+1 : NULL;
	input->file_path = temp;

	title = xmms_get_titlestring(xmms_get_gentitle_format(), input);
	if ( title == NULL )
		title = g_strdup(input->file_name);

	g_free(temp);
	g_free(input);

	return title;
}

static void playfullscreen(char *filename)
{   

    if (nowplaying!=filename)  {
	if (nowplaying) g_free(nowplaying); 
	nowplaying=g_strdup(filename);
    }

    argval[0]="mplayer";
    argval[1]="-framedrop";
    argval[2]="-vop";
    argval[3]="rotate=6,scale";
    argval[4]="-vo";
    argval[5]="fbdev";
    argval[6]="-slave";
    argval[7]="-fs";
    argval[8]="-quiet";
    argval[9]="-v";
    argval[10]=filename;
    argval[11]=NULL;

    if (!seeking) get_current_output_plugin()->open_audio(FMT_U16_NE,44100,OSSCLOSE);

  printf("Starting to play movie \n");
  if (!playing)
  {    
    fprintf(stderr, "Playing Movie very soon\n");
    pipe(filedesin);
    pipe(filedesout);
    if (!fcntl(filedesout[1],F_SETFL,O_NONBLOCK)) printf("O_NONBLOCK set successfully\n");
    else printf("O_NONBLOCK WAS NOT set successfully\n");
    if (!fcntl(filedesout[0],F_SETFL,O_NONBLOCK)) printf("O_NONBLOCK set successfully\n");
    else printf("O_NONBLOCK WAS NOT set successfully\n");
    control=filedesin[1];
    feedback=filedesout[0];

    child=fork();
    if (child==0)
    {
	close(0);
	dup(filedesin[0]);
/*	close(1);
	dup(filedesout[1]);
*/	execvp("mplayer",argval);
    } 
    else assert(child>0);    
    playing=TRUE;
    fprintf(stderr,"Play file fullscreen done \n");
  }
}

static void playwindowed(char *filename)
{    
    if (nowplaying!=filename)  {
	if (nowplaying) g_free(nowplaying); 
	nowplaying=g_strdup(filename);
    }

    argval[0]="mplayer";
    argval[1]="-geometry";
    argval[2]="20:8";
    argval[3]="-framedrop";
    argval[4]="-sws";    
    argval[5]="0";    
    argval[6]="-vop";
    argval[7]="rotate=1,scale=224:-2,rotate=6";
    argval[8]="-slave";
    argval[9]="-quiet";
    argval[10]="-vo";
    argval[11]="fbdev";
    argval[12]="-v";
    argval[13]=filename;
    argval[14]=NULL;

    if (!seeking) get_current_output_plugin()->open_audio(FMT_U16_NE,44100,OSSCLOSE);

  printf("Starting to play movie \n");
  if (!playing)
  {    
    printf("Playing movie very soon\n");
    pipe(filedesin);
    pipe(filedesout);
    if (!fcntl(filedesout[1],F_SETFL,O_NONBLOCK)) printf("O_NONBLOCK set successfully\n");
    else printf("O_NONBLOCK WAS NOT set successfully\n");
    if (!fcntl(filedesout[0],F_SETFL,O_NONBLOCK)) printf("O_NONBLOCK set successfully\n");
    else printf("O_NONBLOCK WAS NOT set successfully\n");
    control=filedesin[1];
    feedback=filedesout[0];

    child=fork();
    if (child==0)
    {
	close(0);
	dup(filedesin[0]);
	close(1);
	dup(filedesout[1]);
	execvp("mplayer",argval);
    } 
    else assert(child>0);    
    playing=TRUE;
    fprintf(stderr,"Play file in window done \n");
  }

}


static void mplay_file(char *filename)
{    
    if (fs) playfullscreen(filename);
    else playwindowed(filename);
}

static void mstop(void)
{
/*  kill(child,2);*/
  char quit[6]={ 'q','u','i','t',0x0a,0 };

  if (playing) {
    printf("writing to filedes: %s\n",quit);
    write(control,quit,5);

    waitpid(child,NULL,0);
    close(control);
    close(feedback);

    playing=FALSE;
    fprintf(stderr,"------- Movie stopped ! \n");
  }
}

static void mwav_pause(short p)
{
/*  printf("writing to filedes %d\n",filedesin[1]);
  fprintf(control,"pause\n");*/
}

static void mseek(int time)
{
    char cb[100];
    int lng;
    if (time>=0) {
	fprintf(stderr,"Really seeking\n");
	lng = sprintf(cb,"seek %d 1\n",time*100/256);
	write(control,cb,lng);    
    } 
    else if (playing) {
	long int pos=spos;
	fprintf(stderr,"Switching screen size and ...\n");
	fprintf(stderr,"Now playing %s \n",nowplaying);
	if (time==-5) {
	    seeking=1;

	}
	else if (time==-10) {
	    seeking=2;	

	}
	fprintf(stderr," -------- Child : %d \n",child);
    }
}

static int readfeedback(){
    int len;
    int c=0;
    
      do {	
        if (((&statusline[RBSIZE-1])-writepos)<0) writepos=statusline;
	len=read(feedback,writepos,(&statusline[RBSIZE])-writepos);
	if (len>0) {
	    writepos+=len;
	    c+=len;
	}
      } while (len>0);
      return c;
}

static char* readbufferln(){
    int ofs=0;
    char *oldrp;
    
    oldrp=readpos;
    if (writepos==readpos) return NULL;
    
    do {
	linebuffer[ofs]=*readpos;
	readpos++;
	ofs++;
	if (((&statusline[RBSIZE-1])-readpos)<0) readpos=statusline;	    
    }	while ((linebuffer[ofs-1]!='\n') && (linebuffer[ofs-1]!=13) && (readpos!=writepos));
    if ((linebuffer[ofs-1]!='\n') && (linebuffer[ofs-1]!=13)) {
	readpos=oldrp;
	return NULL;	
    }
    linebuffer[ofs++]='\n';
    linebuffer[ofs]=0;
    return linebuffer;
}

static int mget_time(void)
{

    long len;
    long tmp1,tmp2;
    char *he,ha[RBSIZE],puf[10];
    int status;
    
    if (seeking) {
	int pos=spos;
	if (seeking==1) {
	    mstop();
	    playfullscreen(nowplaying);	
/*	    sleep(1);*/
/*	    mseek(pos);*/
	    seeking=0;
	}	
	else if (seeking==2) {
	    mstop();
	    playwindowed(nowplaying);	
/*	    sleep(1);*/
/*	    mseek(pos);*/
	    seeking=0;
	}
    }

    if (waitpid(child,&status,WNOHANG)==-1)
    {
/*	if (playing) get_current_output_plugin()->close_audio();  */
	playing=FALSE;
    } else 
    {
	char cb[100];
	int lng;
	lng = sprintf(cb,"get_percent_pos\n");
	write(control,cb,lng);    

	ha[0]=0;
	if (readfeedback()) {
	    do {
		he=readbufferln();
/*		printf("Line: %s",he);*/
		if (he) 
		    strcpy(ha,he);
	    } while (he);	    
/*	    printf("Last Line: %s",ha);		    */
	    if (ha[0]!=0) sscanf(ha, "ANS_PERCENT_POSITION=%d",&spos);
/*	    printf("Stream %d Audio %ld, Video: %ld \n",spos, atime,vtime);*/
	    if (spos>100) spos=1;
	}	
    }
    if (playing) return spos*2560;
    else {
	return -1;    
    }
}

static void mget_song_info(char *filename, char **title, int *length)
{

    (*length)=256000;    
    (*title)=mget_title(filename);
}
