/*  
   xmms-sid - SIDPlay input plugin for X MultiMedia System (XMMS)

   Configuration dialog
   
   Written by Matti "ccr" Hamalainen <mhamalai@ratol.fi>
   (Interface created with Glade, the Gtk+ interface builder,
   code edited by hand afterwards to make it work here)
   
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/
#include "xmms-sid.h"
#include <libxmms/configfile.h>
#include <gtk/gtk.h>
#include <stdio.h>


/*
 * General stuff
 */
static GtkWidget *xs_configwin = NULL;
static GtkWidget *xs_fileselector = NULL;

static GtkObject *cfg_samplerate_adj, *cfg_filt_fm_adj, *cfg_filt_ft_adj, *cfg_filt_fs_adj;
static GtkWidget *cfg_res_8bit, *cfg_res_16bit, *cfg_checkfilecontent, *cfg_chn_mono;
static GtkWidget *cfg_chn_stereo, *cfg_chn_autopan, *cfg_mem_banksw, *cfg_mem_transrom, *cfg_mem_playsid;
static GtkWidget *cfg_songnameformat, *cfg_clock_pal, *cfg_clock_ntsc, *cfg_clock_force, *cfg_ok, *cfg_cancel;
static GtkWidget *cfg_wav_mos6581, *cfg_wav_mos8580, *cfg_emufilters, *cfg_stil_browse, *cfg_stil_use, *cfg_stil_path;



/*
 * Configuration specific stuff
 */
#define XS_CFGTABLE_MAX	15

T_sid_cfg_entry xs_cfgtable[XS_CFGTABLE_MAX] = {
	{ ATYPE_INT,	&xs_cfg.bitsPerSample,	"bitsPerSample",},
	{ ATYPE_INT,	&xs_cfg.channels,	"channels"	},
	{ ATYPE_INT,	&xs_cfg.frequency,	"frequency"	},
	{ ATYPE_BOOL,	&xs_cfg.mos8580,	"mos8580",	},
	{ ATYPE_BOOL,	&xs_cfg.emulateFilter,	"emulateFilter"	},
	{ ATYPE_FLOAT,	&xs_cfg.filterFs,	"filterFs"	},
	{ ATYPE_FLOAT,	&xs_cfg.filterFm,	"filterFm"	},
	{ ATYPE_FLOAT,	&xs_cfg.filterFt,	"filterFt"	},
	{ ATYPE_INT,	&xs_cfg.memoryMode,	"memoryMode"	},
	{ ATYPE_INT,	&xs_cfg.clockSpeed,	"clockSpeed"	},
	{ ATYPE_BOOL,	&xs_cfg.forceSpeed,	"forceSpeed"	},
	{ ATYPE_BOOL,	&xs_cfg.detectMagic,	"detectMagic"	},
	{ ATYPE_STR,	&xs_cfg.stilpath,	"STILpath"	},
	{ ATYPE_BOOL,	&xs_cfg.usestil,	"useSTIL"	},
	{ ATYPE_STR,	&xs_cfg.fileInfo,	"fileInfo"	}
};



/*
 * Get the configuration (from file or default)
 */
void xs_cfg_filter_reset(void)
{
	/* Reset filter values */
	xs_cfg.filterFs = XMMS_SID_DEFAULT_FILTERFS;
	xs_cfg.filterFm = XMMS_SID_DEFAULT_FILTERFM;
	xs_cfg.filterFt = XMMS_SID_DEFAULT_FILTERFT;
}


void xs_get_configure(void)
{
	gchar *cfg_fn, *tmpstr;
	ConfigFile *cfg_file;
	gint i;
	
	/* Pre-initialize configuration structure */
	xs_cfg.bitsPerSample	= 16;
	xs_cfg.channels		= XMMS_SID_CHN_MONO;
	xs_cfg.frequency	= 44100;
	xs_cfg.mos8580		= FALSE;
	xs_cfg.emulateFilter	= TRUE;
	xs_cfg.memoryMode	= XMMS_SID_MPU_BANK_SWITCHING;
	xs_cfg.clockSpeed	= XMMS_SID_CLOCK_PAL;
	xs_cfg.forceSpeed	= FALSE;
	xs_cfg.detectMagic	= FALSE;
	xs_strcalloc(&xs_cfg.stilpath, "~/C64Music/DOCUMENTS/STIL.txt");
	xs_cfg.usestil		= FALSE;
	xs_strcalloc(&xs_cfg.fileInfo, "%1 - %2");

	xs_cfg_filter_reset();

	
	/* Try to open the XMMS configuration file */
	cfg_fn = g_strconcat(g_get_home_dir(), "/.xmms/config", NULL);
	cfg_file = xmms_cfg_open_file(cfg_fn);
	g_free(cfg_fn);
	if (cfg_file == 0) return;

	/* Read the new settings from XMMS configuration file */
	
	/* WARNING: Some "goto magic" coming up! */

	for (i = 0; i < XS_CFGTABLE_MAX; i++) {
	
		switch (xs_cfgtable[i].atype) {
		case ATYPE_INT:		if (!xmms_cfg_read_int(cfg_file, XMMS_SID_CONFIG_IDENT,
					xs_cfgtable[i].aname, (gint *) xs_cfgtable[i].adata)) goto rexit;
					break;
										
		case ATYPE_BOOL:	if (!xmms_cfg_read_boolean(cfg_file, XMMS_SID_CONFIG_IDENT,
					xs_cfgtable[i].aname, (gboolean *) xs_cfgtable[i].adata)) goto rexit;
					break;
					
		case ATYPE_FLOAT:	if (!xmms_cfg_read_float(cfg_file, XMMS_SID_CONFIG_IDENT,
					xs_cfgtable[i].aname, (gfloat *) xs_cfgtable[i].adata)) goto rexit;
					break;
					
		case ATYPE_STR:		if (!xmms_cfg_read_string(cfg_file, XMMS_SID_CONFIG_IDENT,
						xs_cfgtable[i].aname, &tmpstr)) goto rexit;

					/* Read was successfull */
					xs_strcalloc((gchar **) xs_cfgtable[i].adata, tmpstr);

					g_free(tmpstr);
					
					break;

		default:		XSERR("Internal: Unsupported setting type found while reading configuration file. Please report to author!\n");
					break;
		}

		}

rexit:
	/* Free the config file */
	xmms_cfg_free(cfg_file);
}



/*
 * Update filter setting sliders
 */
static void xs_cfg_filt_update(void)
{
}


/*
 * Reset filter settings to defaults
 */
static void xs_cfg_filt_reset(void)
{
	xs_cfg_filter_reset();
	xs_cfg_filt_update();
}


/*
 * STIL-database file selector response-functions
 */




/*
 * Open up a file-selection dialog for browsing STIL-database file
 */


/*
 * Execute the configuration panel
 */
void xs_configure(void)
{
}

