/*  XMMS - Cross-platform multimedia player
 *  Copyright (C) 1998-2001  Peter Alm, Mikael Alm, Olle Hallnas,
 *                           Thomas Nilsson and 4Front Technologies
 *  Copyright (C) 1999-2001  Haavard Kvaalen
 *
 *  remixed by p.nis/dolOps
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
#include "OSS.h"
#include "libxmms/util.h"
#include "xmms/eoss.h"
#include <errno.h>
#include "conv.h"

#define NFRAGS		32

static gint fd = 0, headphonefd = 0, speakerfd = 0;
static char *buffer=NULL;
static gboolean going, prebuffer, paused, unpause, do_pause, remove_prebuffer;
static gint device_buffer_used, real_buffer_size, buffer_size, prebuffer_size, blk_size;
static gint rd_index = 0, wr_index = 0;
static gint output_time_offset = 0;
static guint64 output_bytes = 0;
static gint flush;
static gint fragsize, device_buffer_size=0;
static gchar *device_name;
static pthread_t buffer_thread;
static gboolean select_works;
static gboolean havedevdsp=FALSE;

pthread_mutex_t nss_mutex = PTHREAD_MUTEX_INITIALIZER; 
GList* newsongstart=NULL;
gint songsplayed=0;

format_info input;
format_info internal;
format_info output;

static oss_priv_t *iconvert=0,*oconvert=0;

int current_output=0; //0 not inited 1 headphone 2 speaker

void oss_check_dsp(format_info fmt);

static void oss_calc_device_buffer_used(void)
{
	audio_buf_info buf_info;
	if(paused)
		device_buffer_used = 0;
	else if (!ioctl(fd, SNDCTL_DSP_GETOSPACE, &buf_info))
		device_buffer_used = (buf_info.fragstotal * buf_info.fragsize) - buf_info.bytes;
}

static int oss_calc_bitrate(int oss_fmt, int rate, int channels)
{
	int bitrate = rate * channels;

	if (oss_fmt == AFMT_U16_BE || oss_fmt == AFMT_U16_LE ||
	    oss_fmt == AFMT_S16_BE || oss_fmt == AFMT_S16_LE)
		bitrate *= 2;

	return bitrate;
}

static int oss_get_format(AFormat fmt)
{
	int format = 0;

	switch (fmt)
	{
		case FMT_U8:
			format = AFMT_U8;
			break;
		case FMT_S8:
			format = AFMT_S8;
			break;
		case FMT_U16_LE:
			format = AFMT_U16_LE;
			break;
		case FMT_U16_BE:
			format = AFMT_U16_BE;
			break;
		case FMT_U16_NE:
#ifdef WORDS_BIGENDIAN
			format = AFMT_U16_BE;
#else
			format = AFMT_U16_LE;
#endif
			break;
		case FMT_S16_LE:
			format = AFMT_S16_LE;
			break;
		case FMT_S16_BE:
			format = AFMT_S16_BE;
			break;
		case FMT_S16_NE:
#ifdef WORDS_BIGENDIAN
			format = AFMT_S16_BE;
#else
			format = AFMT_S16_LE;
#endif
			break;
	}

	return format;
}

gint oss_get_written_time(void)
{
    return 0; // only used for add_vis_pcm , which is a dummy fn for us
}

gint oss_get_output_time(void)
{
	guint64 bytes;

	if (!fd || !going)
		return 0;

	bytes = output_bytes < device_buffer_used ?
		0 : output_bytes - device_buffer_used;

	return output_time_offset + ((long long)(bytes * 1000) / (long long)internal.bps);
}

static int oss_used(void)
{
	if (wr_index >= rd_index)
		return wr_index - rd_index;
	return buffer_size - (rd_index - wr_index);
}

gint oss_playing(void)
{
    return FALSE; // hack
}

gint oss_free(void)
{
    long long ret,rdi=rd_index,wri=wr_index;
    if (rdi > wri)
    	ret=(long long)(rdi - wri)*(long long)input.bps/(long long)internal.bps;
    else ret=(long long)(buffer_size - (wri - rdi))*(long long)input.bps/(long long)internal.bps;	    	    	    

    return (int)ret;
}

static inline ssize_t write_all(void *buf, size_t count)
{
	ssize_t done = 0, kount;
	
	oss_check_dsp(internal);

	output_bytes += count;
	kount=convert(oconvert,buf, buf, count);
	
	do {
		ssize_t n = write(fd, buf, kount - done);
		if (n == -1)
		{
			if (errno == EINTR)
				continue;
			else
				break;
		}
		done += n;
	} while (kount > done);

	return count;
}

void oss_write(gpointer ptr, int length)
{
    int cnt, ccnt, off = 0;

    if (!havedevdsp) return;
    
    remove_prebuffer = FALSE;
    while (length > 0)
    {
    	cnt = MIN(length, (long long)(real_buffer_size - wr_index)*(long long)input.bps/((long long)internal.bps + 1LL));
	cnt>>=2;
    	cnt<<=2;  /* For some odd sample rates */
//	printf("becsult kiirando %d  vegeig %d \n",cnt,buffer_size-wr_index);
	ccnt=convert(iconvert,(char *)ptr+off,buffer+wr_index,cnt);
//	printf("Kiirnek %d\n",ccnt);

	if (!ccnt) { 
	    buffer_size=wr_index;
	    wr_index=0;
	}
	else {
	    printf("Kiirt %d\n",ccnt);
	    wr_index = (wr_index + ccnt) % real_buffer_size;
	    length -= cnt;
	    off += cnt;
	}
    }    	
}

void oss_close(void)
{
}


void oss_flush(gint time)
{
    if (!havedevdsp) return;
    flush = time;
    while (flush != -1)
	xmms_usleep(10000);
}

void oss_pause(short p)
{
	if (p == TRUE)
		do_pause = TRUE;
	else
		unpause = TRUE;
}

void *oss_loop(void *arg)
{
	gint length, cnt;
	fd_set set;
	struct timeval tv;
	gint haszn;
	gint boundary;
	gboolean nextsong;

	while (going)
	{
		haszn=oss_used();
		if (haszn > prebuffer_size)
		{
		    prebuffer = FALSE;
		};
		if (oss_used() > 0 && !paused && !prebuffer)
		{
			tv.tv_sec = 0;
			tv.tv_usec = 10000;
			FD_ZERO(&set);
			FD_SET(fd, &set);
			if(!select_works || (select(fd + 1, NULL, &set, NULL, &tv) > 0))
			{
				length = MIN(blk_size, oss_used());
				pthread_mutex_lock(&nss_mutex);
				if (newsongstart!=NULL)
				    boundary=GPOINTER_TO_UINT(g_list_first(newsongstart)->data);
				else boundary=buffer_size+2;
				pthread_mutex_unlock(&nss_mutex);
				nextsong=FALSE;
						
				while (length > 0)
				{
					cnt = MIN(length,buffer_size-rd_index);
					if (boundary>=rd_index && boundary<rd_index+cnt)
					    nextsong=TRUE;
					write_all(buffer + rd_index, cnt);
					rd_index=(rd_index+cnt)%buffer_size;
					length-=cnt;				
				}
				if (!oss_used())
					ioctl(fd, SNDCTL_DSP_POST, 0);
				if (nextsong)
				{
				    songsplayed++;
				    printf(" Songsplayed: %d \n",songsplayed);
				    /* Reset the output time and remove from nextsongstart */
				    output_time_offset = 0;
				    output_bytes = 0;
				    pthread_mutex_lock(&nss_mutex);
				    newsongstart=g_list_remove(newsongstart,GUINT_TO_POINTER(boundary));
				    pthread_mutex_unlock(&nss_mutex);
				}
			}
		}
		else
		{
			xmms_usleep(10000);
		};
		oss_calc_device_buffer_used();
		if (do_pause && !paused)
		{
			do_pause = FALSE;
			paused = TRUE;
			/*
			 * We lose some data here that is sent to the
			 * soundcard, but not yet played.  I don't
			 * think this is worth fixing.
			 */
			ioctl(fd, SNDCTL_DSP_RESET, 0);
		}
		else if (unpause && paused)
		{
			unpause = FALSE;
			paused = FALSE;
		}

		if (flush != -1)
		{
			output_time_offset = flush;
			rd_index = wr_index = output_bytes = 0;
			prebuffer = TRUE;
			if (flush==0)
			{
			    pthread_mutex_lock(&nss_mutex);
			    g_list_free(newsongstart);
			    printf("SONGSPLAYED = 0 \n");
			    newsongstart=NULL;
			    pthread_mutex_unlock(&nss_mutex);
			    songsplayed=0;		
			}    
			flush = -1;
		}
	}

	ioctl(fd, SNDCTL_DSP_RESET, 0);
	close(fd);
	g_free(buffer);
	pthread_exit(NULL);
}

void oss_check_dsp(format_info fmt)
{
	int frag, stereo, ret, cout, tmp;
	struct timeval tv;
	fd_set set;

	if (!headphonefd) {
	    headphonefd = open("/dev/dsp", O_WRONLY);
	    if (headphonefd==-1) {
	        fd=0;
		headphonefd=0;
		return;
	    }
	}

	ioctl(headphonefd, SOUND_MIXER_READ_DEVMASK, &tmp);

	if (!tmp && !speakerfd) {
	    speakerfd = open("/dev/dsp1", O_WRONLY);
	    if (speakerfd==-1) speakerfd=0;
	}
	else if (tmp && speakerfd) {
	    if (speakerfd>0) close(speakerfd);
	    speakerfd=0;
	}

	cout=(tmp || !speakerfd) ? 1:2;
	
	if (cout!=current_output) {
	    fd = (cout==1) ? headphonefd:speakerfd;
	    output=fmt;
	    current_output=cout;
	    ioctl(fd, SNDCTL_DSP_RESET, 0);
	    ioctl(fd, SNDCTL_DSP_SETFMT, &output.format.oss);
	    if (ioctl(fd, SNDCTL_DSP_SETFMT, &output.format.oss) == -1)
		g_warning("SNDCTL_DSP_SETFMT ioctl failed: %s",
			  strerror(errno));

	    stereo = output.channels - 1;
	    ioctl(fd, SNDCTL_DSP_STEREO, &stereo);
	    output.channels=stereo+1;
	    if (ioctl(fd, SNDCTL_DSP_SPEED, &output.frequency) == -1)
		g_warning("SNDCTL_DSP_SPEED ioctl failed: %s", strerror(errno));

	    if (ioctl(fd, SNDCTL_DSP_GETBLKSIZE, &blk_size) == -1)
 		blk_size = 1L << fragsize;

	    output.bps=oss_calc_bitrate(output.format.oss,output.frequency,output.channels);

	    if (oconvert) free(oconvert);		
	    oconvert=convert_init(internal,output);

	/*
	 * Stupid hack to find out if the driver support selects, some
	 * drivers won't work properly without a select and some won't
	 * work with a select :/
	 */
	
	    tv.tv_sec = 0;
    	    tv.tv_usec = 50000;
	    FD_ZERO(&set);
	    FD_SET(fd, &set);
	    ret = select(fd + 1, NULL, &set, NULL, &tv);
	    if (ret > 0)
		select_works = TRUE;
	    else
		select_works = FALSE;
	}
}

gint oss_init_audio(AFormat fmt, gint rate, gint nch)
{
	internal.channels=nch;
	internal.format.oss=oss_get_format(fmt);
	internal.frequency=rate;
	internal.bps=oss_calc_bitrate(internal.format.oss,internal.frequency,internal.channels);
	
	printf("Internal bps %d \n",internal.bps);

	oss_check_dsp(internal);
	output.bps=oss_calc_bitrate(output.format.oss,output.frequency,output.channels);
	
	fprintf(stderr,"bs: %d,%d \n",oss_cfg.buffer_size,internal.bps);
	real_buffer_size = buffer_size = (oss_cfg.buffer_size * internal.bps);
	fprintf(stderr,"Allocating memory soon! %d bytes\n",buffer_size);
	if (buffer_size < 8192)
		buffer_size = 8192;
	prebuffer_size = (buffer_size * oss_cfg.prebuffer) / 100;
	if (buffer_size - prebuffer_size < 4096)
		prebuffer_size = buffer_size - 4096;
		buffer_size += device_buffer_size;
	fprintf(stderr,"Allocating memory! %d bytes\n",buffer_size);
	buffer = g_malloc0(buffer_size);
	fprintf(stderr,"OK! prebuf: %x %d \n",buffer,prebuffer_size);

	flush = -1;
	prebuffer = TRUE;
	wr_index = rd_index = output_time_offset = output_bytes = 0;
	paused = FALSE;
	do_pause = FALSE;
	unpause = FALSE;
	remove_prebuffer = FALSE;

	going = 1;
	pthread_create(&buffer_thread, NULL, oss_loop, NULL);
	pthread_mutex_lock(&nss_mutex);

	g_list_free(newsongstart);
	printf("SONGSPLAYED in open = 0 \n");
	newsongstart=NULL; /*FIXME - why? */
	pthread_mutex_unlock(&nss_mutex);
	songsplayed=0;		
	havedevdsp=TRUE;
	return 1;
}

void oss_deinit_audio() {
    printf("oss_deinit_audio\n");
    going = 0;
    pthread_join(buffer_thread, NULL);
    if (headphonefd>0) close(headphonefd);
    if (speakerfd>0) close(speakerfd);
    current_output=0;
    headphonefd=speakerfd=0;
    havedevdsp=FALSE;
    printf("oss output deinited succesfully!\n");
}

gint oss_open(AFormat fmt, gint rate, gint nch)
{

	if (nch>ECOMMAND)
	  switch (nch)
	  {
	    case SETBOUNDARY:
		if (!havedevdsp) return 1;
		pthread_mutex_lock(&nss_mutex);
		newsongstart=g_list_append(newsongstart,GUINT_TO_POINTER(wr_index));
		pthread_mutex_unlock(&nss_mutex);
		

		return 1;
		break;
	    case GETSONGSPLAYED:

		return songsplayed;
		break;
	    case ENHANCEDMODE:
		return 1;
		break;
	    case BUFFERUSED:

		return oss_used();
		break;
	    case FLUSHED:
		if (!havedevdsp) return TRUE;
		return (newsongstart==NULL && songsplayed==0 && output_bytes==0 && wr_index==rd_index);
		break;
	    case PAUSED:

		return (paused || do_pause);
		break;
	    case OSSEXIT:
		oss_deinit_audio();
		return 0;
		break;
	    case OSSCLOSE:
		if (havedevdsp) oss_deinit_audio();		
/*		oss_pause(TRUE);
		oss_flush(0);
		close(fd);*/
		return 0;
		break;
	    default:
	        return 0;
	  };

	if (!havedevdsp)
	{
	    oss_init_audio(FMT_S16_LE,44100,2);
	}
	input.format.oss = oss_get_format(fmt);
	input.frequency = rate;
	input.channels = nch;
	input.bps = oss_calc_bitrate(oss_get_format(fmt), rate, nch);
	printf("Input bps : %d \n",input.bps);

	if (iconvert) free(iconvert);
	iconvert=convert_init(input,internal);

	paused = FALSE;
	do_pause = FALSE;
	unpause = FALSE;
	remove_prebuffer = FALSE;
	
	going = 1;
	return 1;
}
