/*  XMMS - Cross-platform multimedia player
 *  Copyright (C) 1998-2001  Peter Alm, Mikael Alm, Olle Hallnas,
 *                           Thomas Nilsson and 4Front Technologies
 *  Copyright (C) 1999-2001  Haavard Kvaalen
 *
 *  remixed by p.nis/dolOps
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
#include "OSS.h"
#include "libxmms/util.h"
#include "xmms/eoss.h"
#include <errno.h>
#include <stdio.h>
#include <inttypes.h>

#define STEPACCURACY 11

typedef struct {
    format_info source;
    format_info dest;    
    uint32_t step;
    uint32_t pt;
} oss_priv_t;

static void converttostereo16(char* buffer,long cnt)
{
    int i;
    
    for(i=cnt;i>0;i--,i--)
    {
	buffer[(i<<1)-1]=buffer[i-1];
	buffer[(i<<1)-3]=buffer[i-1];
	buffer[(i<<1)-2]=buffer[i-2];    
	buffer[(i<<1)-4]=buffer[i-2];
    }
}

static void converttomono16(short* buffer,long cn)
{
    int i;
    int cnt=cn/2;
    
    for(i=0;i<cnt/2;i++)
    {
	buffer[i]=(buffer[(i<<1)]>>1)+(buffer[(i<<1)+1]>>1);
    }
}

static uint32_t resample16mo16mo(oss_priv_t * priv,short *ibuf,short * obuf,uint32_t len){
    uint32_t end=(len >> 1)<<STEPACCURACY;
    uint32_t p=0;
    uint32_t pt=priv->pt;
    uint32_t step=priv->step;
    
    while (pt<end){
	obuf[p++]=ibuf[pt>>STEPACCURACY];
	pt+=step;
    }
    priv->pt=pt-end;
    return (p<<1);
}

static uint32_t resample16st16st(oss_priv_t *priv,uint32_t *ibuf,uint32_t * obuf,uint32_t len){
    uint32_t end=(len & 0xfffc)<<(STEPACCURACY-2);
    uint32_t p=0;
    uint32_t pt=priv->pt;
    uint32_t step=priv->step;

    while (pt<end){
	obuf[p++]=ibuf[pt>>STEPACCURACY];
	pt+=step;
    }
    priv->pt=pt-end;
    return (p<<2);
}

static uint32_t resample8mo16mo(oss_priv_t *priv,unsigned char *ibuf,unsigned short * obuf,uint32_t len){
    uint32_t end=len<<(STEPACCURACY);
    uint32_t p=0;
    uint32_t pt=priv->pt;
    uint32_t step=priv->step;

    while (pt<end){
	obuf[p++]=ibuf[pt>>STEPACCURACY]<<8;
	pt+=step;
    }
    priv->pt=pt-end;
    return (p<<1);
}

static uint32_t resample8st16st(oss_priv_t *priv,unsigned char *ibuf,unsigned short * obuf,uint32_t len){
    uint32_t end=len<<(STEPACCURACY);
    uint32_t p=0;
    uint32_t pt=priv->pt;
    uint32_t step=priv->step;

    while (pt<end){
	obuf[p++]=ibuf[pt>>STEPACCURACY]<<8;
	obuf[p++]=ibuf[(pt>>STEPACCURACY)+1]<<8;
	pt+=(step<<1);
    }
    priv->pt=pt-end;
    return (p<<1);
}

static int convert_swap_endian(void *data, int length)
{
	guint16 *ptr = data;
	int i;
	for (i = 0; i < length; i += 2, ptr++)
		*ptr = GUINT16_SWAP_LE_BE(*ptr);

	return i;
}

static int convert_swap_sign_and_endian_to_native(void *data, int length)
{
	guint16 *ptr = data;
	int i;
	for (i = 0; i < length; i += 2, ptr++)
		*ptr = GUINT16_SWAP_LE_BE(*ptr) ^ 1 << 15;
	return i;
}

static int convert_swap_sign16(void *data, int length)
{
	gint16 *ptr = data;
	int i;
	for (i = 0; i < length; i += 2, ptr++)
		*ptr ^= 1 << 15;
	return i;
}

/* convert to S16_LE */
uint32_t convert(oss_priv_t *priv,void *ibuf, void *obuf, uint32_t len){
    uint32_t l;
    
    if (priv->source.channels==1) {
	if ((priv->source.format.oss==AFMT_U8) || (priv->source.format.oss==AFMT_S8)) 
	    l=resample8mo16mo(priv,ibuf,obuf,len);
	else  
	    l=resample16mo16mo(priv,ibuf,obuf,len);    
    }
    else if (priv->source.channels==2) {
	if ((priv->source.format.oss==AFMT_U8) || (priv->source.format.oss==AFMT_S8)) 
	    l=resample8st16st(priv,ibuf,obuf,len);
	else  
	    l=resample16st16st(priv,ibuf,obuf,len);        
    }

    switch (priv->source.format.oss) {
        case AFMT_U8:
    	    break;
        case AFMT_S8:
	    convert_swap_sign16(obuf,l);
    	    break;
        case AFMT_U16_LE: /* FIXME endiannes */
    	    convert_swap_sign16(obuf,l);
            break;
        case AFMT_U16_BE: /*FIXME endiannes */
    	    convert_swap_sign_and_endian_to_native(obuf,l);
    	    break;
	case AFMT_S16_BE: /*FIXME endiannes */
	    convert_swap_endian(obuf,l);
    	    break;
    	case AFMT_S16_LE: /*FIXME endiannes */ 
    	    break;
    	default:
    }		
        
    
    if ((priv->dest.channels==1)&&(priv->source.channels==2)) {
	converttomono16(obuf,l);
	l>>=1;
    }
    if ((priv->dest.channels==2)&&(priv->source.channels==1)) {
	converttostereo16(obuf,l);
	l<<=1;
    }
        
    return l;
}

oss_priv_t * convert_init(format_info input, format_info output){
    oss_priv_t *priv=malloc(sizeof(oss_priv_t));
    memset(priv,0,sizeof(oss_priv_t));
    priv->pt=0;
    priv->step=(long long)(input.frequency<<STEPACCURACY)/(output.frequency);
    printf("STEP %d \n",priv->step);
    priv->source=input;
    priv->dest=output;
    return priv;
}


